#include <getopt.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

/* The name of this program. */
const char* selfName;

void print_usage (FILE* stream, int exit_code)
{
#ifdef DEBUG
  fprintf (stream, "Function print_usage called. Add code for real executions\n") ;
#endif
  exit (exit_code);
}

int main (int argc, char* argv[]) {
  
  int optionID;

  const char* const shortOptions = "hs:v";

  const struct option longOptions[] = {      
    {	"help",  0, NULL, 'h' },
    { "search",  1, NULL, 's' },
    { "verbose", 0, NULL, 'v' },
    { NULL, 0, NULL, 0 } /* MANDATORY End of array. */
  } ;
  const char* searchString = NULL; 
  
  int verbose = 0;
#ifdef DEBUG
  printf ("\tDemo options: -h|--help, -s VAL|--search VAL, -v|--verbose\n") ;
#endif
  selfName = argv[0];
  do 
    {
      optionID = getopt_long (argc, argv, shortOptions, longOptions, NULL);
      switch (optionID)
	{
	case 'h': 
#ifdef DEBUG
	  printf ("\tOption -h|--help selected\n") ;
	  printf ("\n\n\t\tApplication is stopping soon!!!\n\n\n") ;
#endif
	  print_usage (stdout, 0);
	case 's': 
#ifdef DEBUG
	  printf ("\tOption -s|--search selected\n") ;
	  printf ("\t\tOption argument is: %s\n", optarg) ;
#endif
	  /* This option takes an argument... */
	  searchString = optarg;
	  break;
	case 'v': /* -v or --verbose */
#ifdef DEBUG
	  printf ("\tOption -v|--verbose selected\n") ;
	  printf ("\t\tVerbose is ON. Expect for more information!\n") ;
#endif
	  verbose = 1;
	  break;
	case '?': /* The user specified an invalid option. */
#ifdef DEBUG
	  printf ("\tInvalid or incorrect option was specified\n") ;
	  printf ("\t\tAssume help behavior with error code returned\n") ;
#endif
	  print_usage (stderr, 1);
	case -1: 
#ifdef DEBUG
	  printf ("\tOptions processed. Continue with normal operation\n") ;
#endif
	  break;
	default: /* Something else: unexpected. */
#ifdef DEBUG
	  printf ("\tOptions processed. Unexpected situation encountered. Aborting...\n") ;
#endif
	  abort ();
	}
      
    } while (optionID != -1);


#ifdef DEBUG
	  printf ("\tProcessing %d arguments\n", argc-optind) ;
#endif
  if (verbose) {
    int i;
    for (i = optind; i < argc; ++i)
      printf ("\t\tArgument %d is: %s\n", i, argv[i]);
  }
  
  /* The main program goes here. */
  return 0 ;
}
   
